package model

func init() {
	Register(&User{})
}

// User represents a User model
type User struct {
	tableName  struct{} `pg:"user"`
	Id         int      `json:"id" pg:"id,pk"`
	Email      string   `json:"email" binding:"required,validEmailCheck" pg:"email,unique"`
	FullName   string   `json:"fullName" pg:"full_name"`
	Schacc     string   `json:"schacc" pg:"schacc,unique"`
	Password   string   `json:"-" `
	IsApproved bool     `json:"isApproved" pg:"is_approved,use_zero"`
}

func (u *User) Approve() {
	u.IsApproved = true
}

// UserRepo represents user database interface (the repository)
type UserRepo interface {
	List() ([]User, error)
	Update(*User) (*User, error)
	ApproveUser(*User) error
	View(int) (*User, error)
	Create(*User) error
}
