package repository

import (
	"github.com/go-pg/pg/v10"
	"go.uber.org/zap"
	"service-status-backend/model"
)

// NewHistoryRepo returns a new HistoryRepo instance
func NewHistoryRepo(db *pg.DB, log *zap.Logger) *HistoryRepo {
	return &HistoryRepo{db, log}
}

// HistoryRepo is the client for our svc model
type HistoryRepo struct {
	db  *pg.DB
	log *zap.Logger
}

func (h *HistoryRepo) Create(history *model.History) error {
	err := h.db.Model(&model.Service{}).Where("id = ?", history.ServiceId).First()
	if err != nil {
		return err
	}
	_, err = h.db.Model(history).Column("created_at", "description", "service_id").Returning("id").Insert()
	if err != nil {
		h.log.Warn("UserDB Error", zap.Error(err))
	}

	return err
}

func (h *HistoryRepo) Update(history *model.History) (*model.History, error) {
	err := h.db.Model(&model.Service{}).Where("id = ?", history.ServiceId).First()
	if err != nil {
		return history, err
	}
	_, err = h.db.Model(history).Column("created_at", "description", "service_id").WherePK().UpdateNotZero()
	if err != nil {
		h.log.Warn("UserDB Error", zap.Error(err))
	}
	return history, err
}

func (h *HistoryRepo) Delete(id int) error {
	_, err := h.db.Model(&model.History{}).Where("id = ?", id).Delete()
	if err != nil {
		h.log.Warn("UserDB Error", zap.Error(err))
	}

	return err
}
