package repository

import (
	"context"
	"github.com/go-pg/pg/v10"
	"go.uber.org/zap"
	"service-status-backend/apperr"
	"service-status-backend/model"
)

// NewHistoryRepo returns a new HistoryRepo instance
func NewServiceRepo(db *pg.DB, log *zap.Logger) *ServiceRepo {
	return &ServiceRepo{db, log}
}

// HistoryRepo is the client for our svc model
type ServiceRepo struct {
	db  *pg.DB
	log *zap.Logger
}

// View returns single service by ID
func (s *ServiceRepo) View(id int) (*model.Service, error) {
	var service = new(model.Service)
	service.Id = id
	err := s.db.Model(service).Relation("History").WherePK().First()
	if err != nil {
		s.log.Warn("HistoryRepo Error", zap.Error(err))
		return nil, apperr.NotFound
	}
	return service, nil
}

// List returns list of services
func (s *ServiceRepo) List() ([]model.Service, error) {
	var services []model.Service
	q := s.db.Model(&services)
	if err := q.Select(); err != nil {
		s.log.Warn("ServiceDB Error", zap.Error(err))
		return nil, err
	}
	return services, nil
}

func (s *ServiceRepo) Create(service *model.Service) error {
	_, err := s.db.Model(service).Column("name", "description", "is_okay").Returning("id").Insert()
	if err != nil {
		s.log.Warn("UserDB Error", zap.Error(err))
	}

	return err
}

func (s *ServiceRepo) Update(service *model.Service) (*model.Service, error) {
	_, err := s.db.Model(service).Column("name", "description", "is_okay").WherePK().UpdateNotZero()
	if err != nil {
		s.log.Warn("UserDB Error", zap.Error(err))
	}
	return service, err
}

func (s *ServiceRepo) Delete(id int) error {
	deleteErr := s.db.RunInTransaction(context.Background(), func(tx *pg.Tx) error {
		_, err := tx.Model(&model.Service{}).Where("id = ?", id).Delete()
		if err != nil {
			return err
		}
		_, err = tx.Model(&model.History{}).Where("service_id = ?", id).Delete()

		return err
	})

	if deleteErr != nil {
		s.log.Warn("UserDB Error", zap.Error(deleteErr))
	}

	return deleteErr
}
