package repository

import (
	"github.com/go-pg/pg/v10"
	"go.uber.org/zap"
	"service-status-backend/apperr"
	"service-status-backend/model"
	"strconv"
)

// NewHistoryRepo returns a new HistoryRepo instance
func NewUserRepo(db *pg.DB, log *zap.Logger) *UserRepo {
	return &UserRepo{db, log}
}

// HistoryRepo is the client for our user model
type UserRepo struct {
	db  *pg.DB
	log *zap.Logger
}

// View returns single user by ID
func (u *UserRepo) View(id int) (*model.User, error) {
	var user = new(model.User)
	user.Id = id
	err := u.db.Model(user).WherePK().First()
	if err != nil {
		u.log.Warn("HistoryRepo Error", zap.Error(err))
		return nil, apperr.NotFound
	}
	return user, nil
}

func (u *UserRepo) List() ([]model.User, error) {
	var users []model.User
	q := u.db.Model(&users)
	if err := q.Select(); err != nil {
		u.log.Warn("UserDB Error", zap.Error(err))
		return nil, err
	}
	return users, nil
}

func (u *UserRepo) Update(user *model.User) (*model.User, error) {
	_, err := u.db.Model(user).Column("email", "full_name", "schacc", "is_approved").WherePK().UpdateNotZero()
	if err != nil {
		pgErr, ok := err.(pg.Error)
		if ok {
			if pgErr.Field('C') == strconv.Itoa(23505) {
				return user, apperr.New(400, pgErr.Field('D'))
			}
			u.log.Warn("UserDB Error", zap.Error(err))
		}
	}
	return user, err
}

func (u *UserRepo) Create(user *model.User) error {
	_, err := u.db.Model(user).Column("email", "full_name", "schacc", "password").Returning("id").Insert()
	if err != nil {
		pgErr, ok := err.(pg.Error)
		if ok {
			if pgErr.Field('C') == strconv.Itoa(23505) {
				return apperr.New(400, pgErr.Field('D'))
			}
		}
		u.log.Warn("UserDB Error", zap.Error(err))
	}

	return err
}

func (u *UserRepo) ApproveUser(user *model.User) error {
	user.Approve()
	_, err := u.db.Model(user).Column("is_approved").WherePK().Update()
	if err != nil {
		u.log.Warn("HistoryRepo Error", zap.Error(err))
	}
	return err
}
