package request

import (
	"github.com/gin-gonic/gin"
	"net/http"
	"service-status-backend/apperr"
)

type UserRegisterStruct struct {
	Email           string `json:"email" binding:"required,email"`
	FullName        string `json:"fullName" binding:"required"`
	Schacc          string `json:"schacc" binding:"required"`
	Password        string `json:"password" binding:"required"`
	PasswordConfirm string `json:"PasswordConfirm" binding:"required"`
}

func UserCreate(c *gin.Context) (*UserRegisterStruct, error) {
	var r UserRegisterStruct
	if err := c.ShouldBindJSON(&r); err != nil {
		apperr.Response(c, err)
		return nil, err
	}
	if r.Password != r.PasswordConfirm {
		err := apperr.New(http.StatusBadRequest, "passwords do not match")
		c.AbortWithStatusJSON(http.StatusBadRequest, err)
		return nil, err
	}
	return &r, nil
}

type UserUpdateStruct struct {
	Email      string `json:"email" binding:"email"`
	FullName   string `json:"fullName"`
	Schacc     string `json:"schacc"`
	IsApproved bool   `json:"isApproved"`
}

func UserUpdate(c *gin.Context) (*UserUpdateStruct, error) {
	var r UserUpdateStruct
	if err := c.ShouldBindJSON(&r); err != nil {
		apperr.Response(c, err)
		return nil, err
	}
	return &r, nil
}
